/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestDbChecker.java 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.tests.dbchecker;

import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.ow2.jasmine.event.beans.JasmineEventNotification;
import org.ow2.jasmine.event.beans.JasmineEventNotificationDetail;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventNotificationSLBRemote;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

/**
 * Test class that is used to check if some notifications are present in the
 * database. It will use the EJB3 that is making persistent the data
 * 
 * @author Florent Benoit
 */
public class TestDbChecker {

	private JasmineEventNotificationSLBRemote eventNotificationBean = null;

	@BeforeClass
	public void getBean() throws NamingException {
		this.eventNotificationBean = (JasmineEventNotificationSLBRemote) getContext().lookup("db-ejb/notification");
	}

	@Test
	public void checkNotifications() throws Exception {
		JasmineEventNotification[] notifications = eventNotificationBean
				.getEventNotifications(null, null, null, null, null, null,
						null, null);

		Assert.assertNotNull(notifications, "we should have notifications in the database");
		
		Assert.assertTrue(2 == notifications.length, "We should have 2 notifications");
		
		// Now, check that the 2 notifications are valid
		JasmineEventNotification notification1 = notifications[0];
		JasmineEventNotification notification2 = notifications[1];
		
		Assert.assertFalse(notification1.getStatus().booleanValue());
		Assert.assertFalse(notification2.getStatus().booleanValue());
		
		if ("Info".equals(notification1.getLevel())) {
			// check other notification
			Assert.assertEquals("Error", notification2.getLevel());
			
		} else if ("Error".equals(notification1.getLevel())) {
			Assert.assertEquals("Info", notification2.getLevel());
			
		} else {
			throw new Exception("Invalid notifications");
		}

	}
	
	/**
	 * Check that the date is correct
	 */
	@Test(dependsOnMethods={"checkNotifications"})
	public void testContentNotification() {
		JasmineEventNotification[] notifications = eventNotificationBean
		.getEventNotifications(null, null, null, null, null, null,
				null, null);
		
		JasmineEventNotification notif0 = null;
		JasmineEventNotification notif1 = null;
		
		if ("Info".equals(notifications[0].getLevel())) {
			notif0 = notifications[0];
			notif1 = notifications[1];
		} else {
			notif0 = notifications[1];
			notif1 = notifications[0];
		}
		
		// Test date
		Date date = new Date();
		date.setTime(1252480149410l);
		date.equals(notif0.getDate());
		date.equals(notif1.getDate());
		
		// details of notification 0
		// 4 details
		List<JasmineEventNotificationDetail> details = notif0.getDetails();
		Assert.assertEquals(4, details.size());
		JasmineEventNotificationDetail domainDetail = details.get(0);
		JasmineEventNotificationDetail serverDetail = details.get(1);
		JasmineEventNotificationDetail timestampDetail = details.get(2);
		JasmineEventNotificationDetail valueDetail = details.get(3);
		Assert.assertEquals("domain", domainDetail.getLabel());
		Assert.assertEquals("myDomain", domainDetail.getMessage());

		Assert.assertEquals("server", serverDetail.getLabel());
		Assert.assertEquals("florentServer", serverDetail.getMessage());

		Assert.assertEquals("timestamp", timestampDetail.getLabel());
		Assert.assertTrue(timestampDetail.getMessage().contains("Wed Sep 09"));
		Assert.assertTrue(timestampDetail.getMessage().contains("2009"));
		
		Assert.assertEquals("value", valueDetail.getLabel());
		Assert.assertEquals("myValue2", valueDetail.getMessage());
		
		
		// details of notification 1
		// 4 details
		details = notif1.getDetails();
		Assert.assertEquals(4, details.size());
		domainDetail = details.get(0);
		serverDetail = details.get(1);
		timestampDetail = details.get(2);
		valueDetail = details.get(3);
		Assert.assertEquals("domain", domainDetail.getLabel());
		Assert.assertEquals("myDomain", domainDetail.getMessage());

		Assert.assertEquals("server", serverDetail.getLabel());
		Assert.assertEquals("florentServer", serverDetail.getMessage());

		Assert.assertEquals("timestamp", timestampDetail.getLabel());
		Assert.assertTrue(timestampDetail.getMessage().contains("Wed Sep 09"));
		Assert.assertTrue(timestampDetail.getMessage().contains("2009"));
		
		Assert.assertEquals("value", valueDetail.getLabel());
		Assert.assertEquals("myValueSentByDrools", valueDetail.getMessage());
	}
	
	protected Context getContext() throws NamingException {
		Hashtable<String, Object> env = new Hashtable<String, Object>();
		env.put(Context.INITIAL_CONTEXT_FACTORY, org.ow2.easybeans.component.smartclient.spi.SmartContextFactory.class.getName());
		env.put(Context.PROVIDER_URL, "smart://localhost:" + System.getProperty("smartFactory.port", "2503"));
		return new InitialContext(env);
	}
	

}